/**
 * Copyright (C) 2011 Agrosense <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nl.cloudfarming.client.area.field.shape;

import java.awt.Dialog;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.EnumMap;
import java.util.List;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JButton;
import javax.swing.JLabel;
import net.miginfocom.swing.MigLayout;
import nl.cloudfarming.client.area.AreaEvent;
import nl.cloudfarming.client.area.AreaService;
import nl.cloudfarming.client.area.field.AreaFieldModule;
import nl.cloudfarming.client.area.field.FieldAreaService;
import nl.cloudfarming.client.geoviewer.Layer;
import nl.cloudfarming.client.geoviewer.LayerAction;
import nl.cloudfarming.client.geoviewer.event.GeoEvent;
import nl.cloudfarming.client.logging.AppLogFactory;
import nl.cloudfarming.client.logging.AppLogger;
import nl.cloudfarming.client.model.Bean;
import nl.cloudfarming.client.model.Field;
import nl.cloudfarming.client.model.HelpLine;
import nl.cloudfarming.client.util.GeometryUtil;
import org.jdesktop.swingx.JXPanel;
import org.openide.DialogDescriptor;
import org.openide.DialogDisplayer;
import org.openide.nodes.Node;
import org.openide.util.Lookup;
import org.openide.util.NbBundle;
import org.openide.util.Utilities;

/**
 * Action to save the layerobjects on the importlayer
 * @author Timon Veenstra
 */
public class SaveImportLayerAction extends LayerAction {

    private Dialog dlg;
    private static final String SAVE_IMPORT_LAYER_DIALOG_TITLE = NbBundle.getMessage(AreaFieldModule.class, "action.save_import_layer.dialog.title");
    private static final String SAVE_IMPORT_LAYER_ACTION = NbBundle.getMessage(AreaFieldModule.class, "action.save_import_layer.dialog.action_name");
    private static final String SAVE_IMPORT_LAYER_DIALOG_OK = NbBundle.getMessage(AreaFieldModule.class, "action.save_import_layer.dialog.button.ok");
    private static final String SAVE_IMPORT_LAYER_DIALOG_CANCEL = NbBundle.getMessage(AreaFieldModule.class, "action.save_import_layer.dialog.button.cancel");
    private static final String SAVE_IMPORT_LAYER_DIALOG_TEXT = NbBundle.getMessage(AreaFieldModule.class, "action.save_import_layer.dialog.text");
    private static final String SAVE_IMPORT_LAYER_LOG_FINISHED_SUCCESS = NbBundle.getMessage(AreaFieldModule.class, "action.save_import_layer.loginfo.finished_success");
    private static final AppLogger APPLOGGER = AppLogFactory.getLogger(AreaFieldModule.class);
    private static final Logger LOGGER = Logger.getLogger(SaveImportLayerAction.class.getName());
    private AreaService areaService;
    private final ImportDataProvider dataProvider;

    public void setAreaService(AreaService areaService) {
        this.areaService = areaService;
    }

    public SaveImportLayerAction(ImportDataProvider dataProvider) {
        areaService = Lookup.getDefault().lookup(FieldAreaService.class);
        putValue(NAME, SAVE_IMPORT_LAYER_ACTION);
        this.dataProvider = dataProvider;
    }

    void saveImportLayer() {
        //
        // calculate totals per type
        //
        Map<Shape.ShapeType, Integer> totals = new EnumMap<Shape.ShapeType, Integer>(Shape.ShapeType.class);
        int empty = 0;
        for (Shape.ShapeType t : Shape.ShapeType.values()) {
            totals.put(t, 0);
        }

        for (Shape shape:dataProvider.getShapes()){
            Shape.ShapeType st = shape.getType();
            if (st != null) {
                totals.put(st, totals.get(st) + 1);
            } else {
                empty++;
            }            
        }

        List<Bean> shapeBeans = new ArrayList<Bean>();
        for (Shape shape : dataProvider.getShapes()) {
            Bean bean = parseShape(shape);
            if (bean != null){
                shapeBeans.add(bean);
            }else{
                LOGGER.log(Level.WARNING, "failed to create bean for shape {0}", shape);
            }
        }
        areaService.createShapes(shapeBeans);
        
        Layer layer = Utilities.actionsGlobalContext().lookup(Layer.class);
        
        GeoEvent.getProducer().triggerEvent(GeoEvent.DISCARD_LAYER, layer);
        AreaEvent.getProducer().triggerEvent(AreaEvent.IMPORT_SAVED);

        for (Shape.ShapeType shapeType : totals.keySet()) {
            if (totals.get(shapeType) > 0) {
                APPLOGGER.info(NbBundle.getMessage(AreaFieldModule.class, "action.save_import_layer.loginfo.amount", totals.get(shapeType), shapeType.toString()));
            }
        }

        APPLOGGER.info(SAVE_IMPORT_LAYER_LOG_FINISHED_SUCCESS);
    }

    private Bean parseShape(Shape shape) {
        if (shape == null || shape.getType() == null || shape.getGeometry() == null) {
            return null;
        }
        Double acreage = GeometryUtil.getAcreageInHA(GeometryUtil.getAcreageFromGeometry(shape.getGeometry()));
        switch (shape.getType()) {
            case FIELD:
                return new Field.Builder().setWKT(shape.getGeometry().toText()).setName(shape.getName()).setAcreage(acreage).build();
            case HELPLINE:
                return new HelpLine.Builder().setWKT(shape.getGeometry().toText()).setName(shape.getName()).setAcreage(acreage).build();
//            case AB_LINE:
//                return new AbLine.Builder().setWKT(shape.getGeometry().toText()).setName(shape.getName()).setAcreage(shape.getGeometry().getArea()).build();
        }
        return null;
    }

    /**
     * Creates the dialog Ok button
     *
     * @return The OK button + its action
     */
    private JButton getOkButton() {
        JButton jButtonOK = new JButton();
        jButtonOK.setText(SAVE_IMPORT_LAYER_DIALOG_OK);
        jButtonOK.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent ae) {
                dlg.setVisible(false);
                saveImportLayer();
            }
        });
        return jButtonOK;
    }

    /**
     * Creates the dialog cancel button
     * @return The Cancel button + its action
     */
    private JButton getCancelButton() {
        JButton jButtonCancel = new JButton();
        jButtonCancel.setText(SAVE_IMPORT_LAYER_DIALOG_CANCEL);
        jButtonCancel.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent ae) {
                dlg.setVisible(false);
            }
        });
        return jButtonCancel;
    }

    @Override
    public void actionPerformed(ActionEvent e) {
        //
        // calculate totals per type
        //
        Map<Shape.ShapeType, Integer> totals = new EnumMap<Shape.ShapeType, Integer>(Shape.ShapeType.class);
        int empty = 0;
        for (Shape.ShapeType t : Shape.ShapeType.values()) {
            totals.put(t, 0);
        }

        for (Shape shape: dataProvider.getShapes()){
            Shape.ShapeType st = shape.getType();
            if (st != null) {
                totals.put(st, totals.get(st) + 1);
            } else {
                empty++;
            }            
        }
        //
        // populate the panel with the information about what the user is about to save
        //
        JXPanel panel = new JXPanel(new MigLayout("wrap 2"));
        panel.add(new JLabel(SAVE_IMPORT_LAYER_DIALOG_TEXT), "wrap");

        for (Shape.ShapeType st : totals.keySet()) {
            if (totals.get(st) > 0) {
                panel.add(new JLabel(st.toString() + " :"));
                panel.add(new JLabel(totals.get(st) + ""));
            }
        }
        if (empty > 0) {
            panel.add(new JLabel("empty :"));
            panel.add(new JLabel(empty + ""));
        }
        //
        // create the dialog and add the ok and cancel buttons
        //
        DialogDescriptor dd = new DialogDescriptor(panel, SAVE_IMPORT_LAYER_DIALOG_TITLE);
        dd.setOptions(new Object[]{getOkButton(), getCancelButton()});
        dlg = DialogDisplayer.getDefault().createDialog(dd);
        dlg.setVisible(true);
    }
}
